@@ -3,7 +3,8 @@ import os
 import pytest
 from vito.imutils import flip_layers, imread, imsave, apply_on_bboxes, \
     ndarray2memory_file, memory_file2ndarray, roi, crop, pad, rgb2gray, \
-    grayscale, pixelate, gaussian_blur, set_to, ensure_c3
+    grayscale, pixelate, gaussian_blur, set_to, ensure_c3, concat, \
+    rotate90, rotate180, rotate270, noop
 from vito.pyutils import safe_shell_output
 
 
@@ -140,14 +141,14 @@ def test_imsave(tmp_path):
     assert np.all(img_in[:] == img_out[:])
     ##########################################################################
     # Test 1-bit PNG (requires specifying the mode!)
-    img_in = imread(os.path.join(exdir, 'space-invader.png'), mode='L').astype(np.bool)
+    img_in = imread(os.path.join(exdir, 'space-invader.png'), mode='L').astype(bool)
     assert img_in.ndim == 2 or img_in.shape[2] == 1
-    assert img_in.dtype == np.bool
+    assert img_in.dtype == bool
     imsave(out_fn, img_in)
     _, finfo = safe_shell_output('file', out_fn)
     assert (finfo.split(':')[1].strip() == 'PNG image data, 200 x 149, 1-bit colormap, non-interlaced' or
             finfo.split(':')[1].strip() == 'PNG image data, 200 x 149, 1-bit grayscale, non-interlaced')
-    img_out = imread(out_fn, mode='L').astype(np.bool)
+    img_out = imread(out_fn, mode='L').astype(bool)
     assert img_out.ndim == 2 or img_out.shape[2] == 1
     assert np.all(img_in[:] == img_out[:])
 
@@ -458,3 +459,135 @@ def test_ensure_c3():
         c3 = ensure_c3(x)
         assert c3.ndim == 3 and c3.shape[2] == 3
         assert np.array_equal(x[:, :, :3], c3)
+
+
+def test_concat():
+    # Invalid inputs
+    x = np.zeros((3,5))
+    y = np.ones((5,7))
+    assert concat(None, y, True) is None
+    assert concat(x, None, False) is None
+    with pytest.raises(ValueError):
+        concat(x, y, True)
+    with pytest.raises(ValueError):
+        concat(x, y, False)
+    
+    # ndim == 2, different dtype
+    x = np.random.randint(0, 255, (5, 17), dtype=np.uint8)
+    y = np.random.randint(0, 255, (5, 2), dtype=np.int32)
+    c = concat(x, y, True)
+    # The concatenation should have the "better" dtype of the two inputs
+    assert c.dtype == y.dtype
+    assert np.array_equal(x.astype(c.dtype), c[:, :x.shape[1]])
+    assert np.array_equal(y, c[:, x.shape[1]:])
+
+    # ndim == 2, same dtype
+    y = np.random.randint(0, 255, (5, 2), dtype=x.dtype)
+    c = concat(x, y, True)
+    assert c.dtype == x.dtype
+    assert np.array_equal(x, c[:, :x.shape[1]])
+    assert np.array_equal(y, c[:, x.shape[1]:])
+
+    # ndim == 2 vs ndim == 3, single channel
+    x = np.random.randint(0, 255, (5, 2, 1), dtype=np.uint8)
+    y = np.random.randint(0, 255, (3, 2), dtype=np.int32)
+    c = concat(x, y, False)
+    assert c.dtype == y.dtype
+    assert c.ndim == 3 and c.shape[2] == 1
+    assert np.array_equal(x.astype(c.dtype), c[:x.shape[0], :, :])
+    assert np.array_equal(y[:, :], c[x.shape[0]:, :, 0])
+
+    # ndim == 2 vs ndim == 3, single vs multi-channel (compatible)
+    x = np.random.randint(0, 255, (5, 2), dtype=np.int32)
+    y = np.random.randint(0, 255, (3, 2, 4), dtype=np.uint8)
+    c = concat(x, y, False)
+    assert c.dtype == x.dtype
+    assert c.ndim == 3 and c.shape[2] == y.shape[2]
+    for l in range(c.shape[2]):
+        assert np.array_equal(x[:, :], c[:x.shape[0], :, l])
+    assert np.array_equal(y[:, :, :], c[x.shape[0]:, :, :])
+
+    # ndim == 3, multi-channel
+    # --> Incompatible inputs
+    x = np.random.randint(0, 255, (5, 2, 2), dtype=np.uint8)
+    y = np.random.randint(0, 255, (3, 2, 3), dtype=np.int32)
+    with pytest.raises(ValueError):
+        concat(x, y, False)
+    # --> same number of channels
+    x = np.random.randint(0, 255, (6, 2, 3), dtype=np.uint64)
+    y = np.random.randint(0, 255, (6, 9, 3), dtype=np.int32)
+    c = concat(x, y, True)
+    assert c.dtype == x.dtype
+    assert np.array_equal(x, c[:, :x.shape[1], :])
+    assert np.array_equal(y, c[:, x.shape[1]:, :])
+    # --> different number of channels (but compatible)
+    x = np.random.randint(0, 255, (6, 2, 1), dtype=np.uint64)
+    y = np.random.randint(0, 255, (6, 9, 3), dtype=np.int32)
+    c = concat(x, y, True)
+    assert c.dtype == x.dtype
+    for l in range(3):
+        assert np.array_equal(x[:, :, 0], c[:, :x.shape[1], l])
+    assert np.array_equal(y.astype(c.dtype), c[:, x.shape[1]:])
+    # ... switch parameter order
+    c = concat(y, x, True)
+    assert c.dtype == x.dtype
+    assert np.array_equal(y.astype(c.dtype), c[:, :y.shape[1]])
+    for l in range(3):
+        assert np.array_equal(x[:, :, 0], c[:, y.shape[1]:, l])
+
+
+def test_noop():
+    for x in [None, 1, 3.7, np.zeros((3, 2)), 'test']:
+        if isinstance(x, np.ndarray):
+            assert np.array_equal(noop(x), x)
+        else:
+            assert noop(x) == x
+
+
+def test_rotation():
+    # Invalid inputs
+    assert rotate90(None) is None
+    assert rotate180(None) is None
+    assert rotate270(None) is None
+
+    for x in [np.array([[1, 2], [3, 4]], dtype=np.int32),
+              np.random.randint(0, 255, (2, 2, 1), dtype=np.uint8),
+              np.random.randint(0, 255, (2, 2, 3), dtype=np.uint8),
+              np.random.randint(0, 255, (2, 2, 3), dtype=np.int64)]:
+        if x.ndim == 2:
+            y = rotate90(x)
+            assert y[0, 0] == x[0, 1] and \
+                y[0, 1] == x[1, 1] and \
+                y[1, 0] == x[0, 0] and \
+                y[1, 1] == x[1, 0]
+
+            y = rotate180(x)
+            assert y[0, 0] == x[1, 1] and \
+                y[0, 1] == x[1, 0] and \
+                y[1, 0] == x[0, 1] and \
+                y[1, 1] == x[0, 0]
+
+            y = rotate270(x)
+            assert y[0, 0] == x[1, 0] and \
+                y[0, 1] == x[0, 0] and \
+                y[1, 0] == x[1, 1] and \
+                y[1, 1] == x[0, 1]
+        else:
+            for l in range(x.shape[2]):
+                y = rotate90(x)
+                assert y[0, 0, l] == x[0, 1, l] and \
+                    y[0, 1, l] == x[1, 1, l] and \
+                    y[1, 0, l] == x[0, 0, l] and \
+                    y[1, 1, l] == x[1, 0, l]
+
+                y = rotate180(x)
+                assert y[0, 0, l] == x[1, 1, l] and \
+                    y[0, 1, l] == x[1, 0, l] and \
+                    y[1, 0, l] == x[0, 1, l] and \
+                    y[1, 1, l] == x[0, 0, l]
+
+                y = rotate270(x)
+                assert y[0, 0, l] == x[1, 0, l] and \
+                    y[0, 1, l] == x[0, 0, l] and \
+                    y[1, 0, l] == x[1, 1, l] and \
+                    y[1, 1, l] == x[0, 1, l]
